package mv3302;

import mv3302.sensor.SimpleConstantTimeSensor;
import simkit.SimEntityBase;
import simkit.smd.Mover;

/**
 * The SimpleConstantTimeMediator class represents a mediator between a target
 * Mover object and a SimpleConstantTimeSensor object. It extends the
 * SimEntityBase class, which provides basic simulation entity functionality.
 */
public class SimpleConstantTimeMediator extends SimEntityBase {

    /**
     * Performs the necessary actions when the target Mover enters the range of
     * the sensor.
     *
     * @param target The Mover object that entered the range of the sensor.
     * @param sensor The SimpleConstantTimeSensor object responsible for
     * detecting the target.
     */
    public void doEnterRange(Mover target, SimpleConstantTimeSensor sensor) {
        if (!sensor.getContacts().contains(target)) {
            sensor.waitDelay("Detection", sensor.getTimeToDetect(), target);
        }
    }

    /**
     * Performs the necessary actions when the target Mover exits the range of
     * the sensor.
     *
     * @param target The Mover object that exited the range of the sensor.
     * @param sensor The SimpleConstantTimeSensor object responsible for
     * detecting the target.
     */
    public void doExitRange(Mover target, SimpleConstantTimeSensor sensor) {
        if (!sensor.getContacts().contains(target)) {
            sensor.waitDelay("Undetection", 0.0, target);
        }
    }
}
