package mv3302;

/**
 * The StoppedPartArrivalProcess class extends the PartArrivalProcess class and
 * represents a process that stops the arrival of parts for a specified
 * duration. It overrides the doRun() method to wait for a specified stop time
 * before resuming the arrival of parts. It also provides methods to schedule
 * the next arrival and get/set the stop time.
 *
 * @author dansl
 */
public final class StoppedPartArrivalProcess extends PartArrivalProcess {

    private int stopTime;

    /**
     * Zero-argument constructor
     */
    public StoppedPartArrivalProcess() {
        super();
    }

    /**
     * Constructs a StoppedPartArrivalProcess object with the given stop time.
     *
     * @param stopTime The duration for which the arrival of parts should be
     * stopped.
     */
    public StoppedPartArrivalProcess(int stopTime) {
        this.stopTime = stopTime;
    }

    /**
     * Overrides the doRun() method from the parent class. Pauses the arrival of
     * parts for the specified stop time.
     */
    @Override
    public void doRun() {
        waitDelay("StopArrivals", getStopTime());
    }

    /**
     * Schedule the next arrival after stopping. This method schedules the
     * Arrival(index) process.
     */
    public void doStopArrivals() {
        interrupt("Arrival");
    }

    /**
     * @return the stopTime
     */
    public int getStopTime() {
        return stopTime;
    }

    /**
     * @param stopTime the stopTime to set
     */
    public void setStopTime(int stopTime) {
        this.stopTime = stopTime;
    }
}
