/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Main.java to edit this template
 */
package mv3302.run;

import mv3302.ShipArrivalProcess;
import mv3302.TwoCranesBerth;
import simkit.Adapter;
import simkit.Schedule;
import simkit.random.RandomVariate;
import simkit.random.RandomVariateFactory;
import simkit.stat.CollectionSizeTimeVaryingStats;
import simkit.stat.SimpleStatsTally;

/**
 * A test class that simulates the arrival and unloading of ships at a two-crane
 * berth. It initializes the required objects, listens to the events triggered
 * by these objects, and collects statistical data during the simulation.
 *
 * @author dansl
 */
public class testShipArrivalProcess {

    /**
     * Runs the simulation of ship arrivals and unloading at a two-crane berth,
     * and prints the collected statistical data at the end of the simulation.
     */
    public static void main(String[] args) {
        RandomVariate interarrivalTimeGenerator = RandomVariateFactory.getInstance("Exponential", .7);
        RandomVariate unloadingTimeGenerator = RandomVariateFactory.getInstance("Uniform", 0.5, 1.5);
        ShipArrivalProcess shipArrivalProcess = new ShipArrivalProcess(interarrivalTimeGenerator, unloadingTimeGenerator);
        System.out.println(shipArrivalProcess);

        TwoCranesBerth twoCranesBerth = new TwoCranesBerth();
        System.out.println(twoCranesBerth);

        //        TwoCranesBerth will "hear" ShipArrivalProcess(Arrival) events
        shipArrivalProcess.addSimEventListener(twoCranesBerth);

//        The twoCranesBerth will "hear" the ShipArrival events of the
//        first as Arrival events
        Adapter adapter = new Adapter("ShipArrival", "Arrival");
        adapter.connect(shipArrivalProcess, twoCranesBerth);

//        This was used for debugging model
//        SimplePropertyDumper simplePropertyDumper = new SimplePropertyDumper(true);
//
//        twoCranesBerth.addPropertyChangeListener(simplePropertyDumper);
//        shipArrivalProcess.addPropertyChangeListener(simplePropertyDumper);
        //        Tally statistics are computed in the same way by "listening" 
        SimpleStatsTally timeInSystemStat = new SimpleStatsTally("timeInSystem");
        twoCranesBerth.addPropertyChangeListener(timeInSystemStat);

        SimpleStatsTally delayInQueueStat = new SimpleStatsTally("delayInQueue");
        twoCranesBerth.addPropertyChangeListener(delayInQueueStat);

        CollectionSizeTimeVaryingStats numberInBerthsStat = new CollectionSizeTimeVaryingStats("berths");
        twoCranesBerth.addPropertyChangeListener(numberInBerthsStat);

        CollectionSizeTimeVaryingStats numberInQueueStat = new CollectionSizeTimeVaryingStats("queue");
        twoCranesBerth.addPropertyChangeListener(numberInQueueStat);
        //        Was "true" when debugging model
        Schedule.setVerbose(false);
        double stopTime = 3650.0;
        Schedule.stopAtTime(stopTime);

//        Initialized & run 
        Schedule.reset();
        Schedule.startSimulation();

        System.out.printf("%nSimulation ended at time %,.2f%n%n", Schedule.getSimTime());

        System.out.printf("Number of Ships Arriving:\t %,d%n", shipArrivalProcess.getNumberArrivals());
        System.out.printf("Number of Ships Unloaded:\t %,d%n", timeInSystemStat.getCount());
        System.out.printf("Maximum # in Queue:\t\t %,.0f%n", numberInQueueStat.getMaxObs());
        System.out.printf("Average # in Queue:\t\t %,.4f%n", numberInQueueStat.getMean());
        System.out.printf("Average # Busy Berths:\t\t %,.4f%n", numberInBerthsStat.getMean());
        System.out.printf("Average Time in System:\t\t %,.4f%n", timeInSystemStat.getMean());
        System.out.printf("Average Delay in Queue:\t\t %,.4f%n", delayInQueueStat.getMean());
    }

}
